package org.openjdk.gcbench.colls;

import org.openjdk.jmh.annotations.*;
import org.openjdk.jmh.infra.Blackhole;

import java.util.Map;
import java.util.TreeMap;
import java.util.concurrent.TimeUnit;

@Warmup(iterations = 5, time = 1, timeUnit = TimeUnit.SECONDS)
@Measurement(iterations = 5, time = 1, timeUnit = TimeUnit.SECONDS)
@Fork(1)
@BenchmarkMode(Mode.AverageTime)
@OutputTimeUnit(TimeUnit.MICROSECONDS)
@Threads(1)
@State(Scope.Benchmark)
public class TreeMapTraversal {

    Map<Integer, Object> map;

    @Param({"1", "10", "100", "1000", "10000", "100000", "1000000"})
    private int size;

    @Param({"0", "1", "10", "100"})
    private int burn;

    @Setup
    public void setup() {
        map = new TreeMap<>();
        for (int c = 0; c < size; c++) {
            map.put(c, new Object());
        }
    }

    @Benchmark
    public void test() {
        for (Object o : map.entrySet()) {
            process(o);
        }
    }

    @CompilerControl(CompilerControl.Mode.DONT_INLINE)
    private void process(Object o) {
        Blackhole.consumeCPU(burn);
    }

}
